# -*- coding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import csv
import os


class CSVPublisher(object):

    def __init__(self):
        self.filepath = None

    def set_filepath(self, filename):
        self.filepath = "/tmp/check_" + str(
           filename) + ".csv"

    def set_csv_fields(self, *args):
        self.fields = args

    def publish(self, *args):
        if os.path.isfile(self.filepath):
            os.remove(self.filepath)
        with open(self.filepath, mode='a') as csv_file:
            if self.fields:
                publisher = csv.DictWriter(
                    csv_file, fieldnames=self.fields)
                publisher.writeheader()
            else:
                publisher = csv.writer(
                    csv_file,
                    delimiter=',',
                    quotechar='"',
                    quoting=csv.QUOTE_MINIMAL)
            for res in args:
                publisher.writerow(res)
