# -*- coding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import json
import os
import shutil


class FilePublisher(object):

    def __init__(self, dirctory):
        self.path = "/tmp/check_" + str(dirctory)
        if os.path.isdir(self.path):
            shutil.rmtree(self.path)
        os.mkdir(self.path)

    def publish(self, filename, context):
        file_path = self.path + "/" + filename
        with open(file_path, 'a') as f:
            json.dump(context, f, indent=2, sort_keys=True)
